@@ -39,19 +39,15 @@
 
 namespace aria2 {
 
-class BtAllowedFastMessage;
-
-typedef SharedHandle<BtAllowedFastMessage> BtAllowedFastMessageHandle;
-
 class BtAllowedFastMessage : public IndexBtMessage {
 public:
   BtAllowedFastMessage(size_t index = 0);
 
   static const uint8_t ID = 17;
 
-  static const std::string NAME;
+  static const char NAME[];
 
-  static SharedHandle<BtAllowedFastMessage> create
+  static BtAllowedFastMessage* create
   (const unsigned char* data, size_t dataLength);
 
   virtual void doReceivedAction();
