@@ -48,24 +48,24 @@ class ChunkChecksum {
 private:
   std::string hashType_;
   std::vector<std::string> pieceHashes_;
-  size_t pieceLength_;
+  int32_t pieceLength_;
 public:
   ChunkChecksum();
 
   ChunkChecksum
   (const std::string& hashType,
    const std::vector<std::string>& pieceHashes,
-   size_t pieceLength);
+   int32_t pieceLength);
 
   bool validateChunk(const std::string& actualDigest,
                      size_t index) const;
 
-  uint64_t getEstimatedDataLength() const;
+  int64_t getEstimatedDataLength() const;
 
   size_t countPieceHash() const;
 
   const std::string& getPieceHash(size_t index) const;
-  
+
   void setPieceHashes(const std::vector<std::string>& pieceHashes);
   const std::vector<std::string>& getPieceHashes() const
   {
@@ -78,12 +78,12 @@ public:
     return hashType_;
   }
 
-  size_t getPieceLength() const
+  int32_t getPieceLength() const
   {
     return pieceLength_;
   }
 
-  void setPieceLength(size_t length)
+  void setPieceLength(int32_t length)
   {
     pieceLength_ = length;
   }
