@@ -52,6 +52,7 @@ class HttpHeaderProcessor;
 class Option;
 class Segment;
 class SocketCore;
+class SocketRecvBuffer;
 
 class HttpRequestEntry {
 private:
@@ -73,13 +74,13 @@ public:
   }
 };
 
-typedef SharedHandle<HttpRequestEntry> HttpRequestEntryHandle;
-typedef std::deque<HttpRequestEntryHandle> HttpRequestEntries;
+typedef std::deque<SharedHandle<HttpRequestEntry> > HttpRequestEntries;
 
 class HttpConnection {
 private:
   cuid_t cuid_;
   SharedHandle<SocketCore> socket_;
+  SharedHandle<SocketRecvBuffer> socketRecvBuffer_;
   SocketBuffer socketBuffer_;
   const Option* option_;
 
@@ -87,7 +88,10 @@ private:
 
   std::string eraseConfidentialInfo(const std::string& request);
 public:
-  HttpConnection(cuid_t cuid, const SharedHandle<SocketCore>& socket);
+  HttpConnection
+  (cuid_t cuid,
+   const SharedHandle<SocketCore>& socket,
+   const SharedHandle<SocketRecvBuffer>& socketRecvBuffer);
   ~HttpConnection();
 
   /**
@@ -109,10 +113,10 @@ public:
    * object which contains response header and HttpRequestHandle object
    * for this response.
    * If a response is not fully received, received header is buffured
-   * in this object and returns 0. 
+   * in this object and returns 0.
    * You should continue to call this method until whole response header is
    * received and this method returns non-null HttpResponseHandle object.
-   * 
+   *
    * @return HttpResponse or 0 if whole response header is not received
    */
   SharedHandle<HttpResponse> receiveResponse();
@@ -124,9 +128,12 @@ public:
   bool sendBufferIsEmpty() const;
 
   void sendPendingData();
-};
 
-typedef SharedHandle<HttpConnection> HttpConnectionHandle;
+  const SharedHandle<SocketRecvBuffer>& getSocketRecvBuffer() const
+  {
+    return socketRecvBuffer_;
+  }
+};
 
 } // namespace aria2
 
