@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_SOCKET_CORE_H_
-#define _D_SOCKET_CORE_H_
+#ifndef D_SOCKET_CORE_H
+#define D_SOCKET_CORE_H
 
 #include "common.h"
 
@@ -44,11 +44,11 @@
 
 #include "a2netcompat.h"
 
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_OPENSSL
 // for SSL
 # include <openssl/ssl.h>
 # include <openssl/err.h>
-#endif // HAVE_LIBSSL
+#endif // HAVE_OPENSSL
 #ifdef HAVE_LIBGNUTLS
 # include <gnutls/gnutls.h>
 #endif // HAVE_LIBGNUTLS
@@ -76,7 +76,7 @@ private:
 
   static int protocolFamily_;
 
-  static std::vector<std::pair<struct sockaddr_storage, socklen_t> > bindAddrs_;
+  static std::vector<std::pair<sockaddr_union, socklen_t> > bindAddrs_;
 
   bool blocking_;
   int secure_;
@@ -85,25 +85,20 @@ private:
   bool wantWrite_;
 
 #if ENABLE_SSL
-  static SharedHandle<TLSContext> tlsContext_;
+  // TLS context for client side
+  static SharedHandle<TLSContext> clTlsContext_;
+  // TLS context for server side
+  static SharedHandle<TLSContext> svTlsContext_;
 #endif // ENABLE_SSL
 
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_OPENSSL
   // for SSL
   SSL* ssl;
 
   int sslHandleEAGAIN(int ret);
-#endif // HAVE_LIBSSL
+#endif // HAVE_OPENSSL
 #ifdef HAVE_LIBGNUTLS
   gnutls_session_t sslSession_;
-  char* peekBuf_;
-  size_t peekBufLength_;
-  size_t peekBufMax_;
-
-  size_t shiftPeekData(char* data, size_t len);
-  void addPeekData(char* data, size_t len);
-  ssize_t gnutlsRecv(char* data, size_t len);
-  ssize_t gnutlsPeek(char* data, size_t len);
 
   void gnutlsRecordCheckDirection();
 #endif // HAVE_LIBGNUTLS
@@ -114,6 +109,16 @@ private:
 
   void setSockOpt(int level, int optname, void* optval, socklen_t optlen);
 
+#ifdef ENABLE_SSL
+  /**
+   * Makes this socket secure. The connection must be established
+   * before calling this method.
+   *
+   * If you are going to verify peer's certificate, hostname must be supplied.
+   */
+  bool tlsHandshake(TLSContext* tlsctx, const std::string& hostname);
+#endif // ENABLE_SSL
+
   SocketCore(sock_t sockfd, int sockType);
 public:
   SocketCore(int sockType = SOCK_STREAM);
@@ -132,13 +137,14 @@ public:
   void joinMulticastGroup
   (const std::string& multicastAddr, uint16_t multicastPort,
    const std::string& localAddr);
-  
+
+  // Enables TCP_NODELAY socket option if f == true.
+  void setTcpNodelay(bool f);
+
   void create(int family, int protocol = 0);
 
   void bindWithFamily(uint16_t port, int family, int flags = AI_PASSIVE);
 
-  void bind(const std::string& addr, uint16_t port, int flags = AI_PASSIVE);
-
   /**
    * Creates a socket and bind it with locahost's address and port.
    * flags is set to struct addrinfo's ai_flags.
@@ -147,6 +153,9 @@ public:
    */
   void bind(uint16_t port, int flags = AI_PASSIVE);
 
+  void bind
+  (const char* addrp, uint16_t port, int family, int flags = AI_PASSIVE);
+
   /**
    * Listens form connection on it.
    * Call bind(uint16_t) before calling this function.
@@ -154,23 +163,42 @@ public:
   void beginListen();
 
   /**
-   * Stores host address and port of this socket to addrinfo.
+   * Stores host address and port of this socket to addrinfo and
+   * returns address family.
+   *
    * @param addrinfo placeholder to store host address and port.
    */
-  void getAddrInfo(std::pair<std::string, uint16_t>& addrinfo) const;
+  int getAddrInfo(std::pair<std::string, uint16_t>& addrinfo) const;
 
   /**
-   * Stores peer's address and port to peerinfo.
+   * Stores address of this socket to sockaddr.  len must be
+   * initialized to the size of sockaddr.  On success, address data is
+   * stored in sockaddr and actual size of address structure is stored
+   * in len.
+   */
+  void getAddrInfo
+  (sockaddr_union& sockaddr, socklen_t& len) const;
+
+  /**
+   * Returns address family of this socket.
+   * The socket must be connected or bounded to address.
+   */
+  int getAddressFamily() const;
+
+  /**
+   * Stores peer's address and port to peerinfo and returns address
+   * family.
+   *
    * @param peerinfo placeholder to store peer's address and port.
    */
-  void getPeerInfo(std::pair<std::string, uint16_t>& peerinfo) const;
+  int getPeerInfo(std::pair<std::string, uint16_t>& peerinfo) const;
 
   /**
    * Accepts incoming connection on this socket.
    * You must call beginListen() before calling this method.
-   * @return accepted socket. The caller must delete it after using it.
+   * @return accepted socket.
    */
-  SocketCore* acceptConnection() const;
+  SharedHandle<SocketCore> acceptConnection() const;
 
   /**
    * Connects to the server named host and the destination port is port.
@@ -278,32 +306,18 @@ public:
     return readDataFrom(reinterpret_cast<char*>(data), len, sender);
   }
 
-  /**
-   * Reads up to len bytes from this socket, but bytes are not removed from
-   * this socket.
-   * This method internally calls isReadable(). The parameter timeout is used
-   * for this method call.
-   * @param data holder to store data.
-   * @param len the maximum size data can store. This method assigns
-   * the number of bytes read to len.
-   */
-  void peekData(char* data, size_t& len);
+#ifdef ENABLE_SSL
+  // Performs TLS server side handshake. If handshake is completed,
+  // returns true. If handshake has not been done yet, returns false.
+  bool tlsAccept();
 
-  void peekData(unsigned char* data, size_t& len)
-  {
-    peekData(reinterpret_cast<char*>(data), len);
-  }
-
-  /**
-   * Makes this socket secure.
-   * If the system has not OpenSSL, then this method do nothing.
-   * connection must be established  before calling this method.
-   *
-   * If you are going to verify peer's certificate, hostname must be supplied.
-   */
-  bool initiateSecureConnection(const std::string& hostname="");
-
-  void prepareSecureConnection();
+  // Performs TLS client side handshake. If handshake is completed,
+  // returns true. If handshake has not been done yet, returns false.
+  //
+  // If you are going to verify peer's certificate, hostname must be
+  // supplied.
+  bool tlsConnect(const std::string& hostname);
+#endif // ENABLE_SSL
 
   bool operator==(const SocketCore& s) {
     return sockfd_ == s.sockfd_;
@@ -333,7 +347,8 @@ public:
   bool wantWrite() const;
 
 #ifdef ENABLE_SSL
-  static void setTLSContext(const SharedHandle<TLSContext>& tlsContext);
+  static void setClientTLSContext(const SharedHandle<TLSContext>& tlsContext);
+  static void setServerTLSContext(const SharedHandle<TLSContext>& tlsContext);
 #endif // ENABLE_SSL
 
   static void setProtocolFamily(int protocolFamily)
@@ -352,7 +367,7 @@ public:
   static void bindAddress(const std::string& iface);
 
   friend void getInterfaceAddress
-  (std::vector<std::pair<struct sockaddr_storage, socklen_t> >& ifAddrs,
+  (std::vector<std::pair<sockaddr_union, socklen_t> >& ifAddrs,
    const std::string& iface, int family, int aiFlags);
 };
 
@@ -374,9 +389,40 @@ int callGetaddrinfo
 // to collect using family argument. aiFlags is passed to
 // getaddrinfo() as hints.ai_flags. No throw.
 void getInterfaceAddress
-(std::vector<std::pair<struct sockaddr_storage, socklen_t> >& ifAddrs,
+(std::vector<std::pair<sockaddr_union, socklen_t> >& ifAddrs,
  const std::string& iface, int family = AF_UNSPEC, int aiFlags = 0);
 
+// Provides functionality of inet_ntop using getnameinfo.  The return
+// value is the exact value of getnameinfo returns. You can get error
+// message using gai_strerror(3).
+int inetNtop(int af, const void* src, char* dst, socklen_t size);
+
+// Provides functionality of inet_pton using getBinAddr.  If af is
+// AF_INET, dst is assumed to be the pointer to struct in_addr.  If af
+// is AF_INET6, dst is assumed to be the pointer to struct in6_addr.
+//
+// This function returns 0 if it succeeds, or -1.
+int inetPton(int af, const char* src, void* dst);
+
+namespace net {
+
+// Stores binary representation of IP address ip which is represented
+// in text.  ip must be numeric IPv4 or IPv6 address. dest must be
+// allocated by caller before the call. For IPv4 address, dest must be
+// at least 4. For IPv6 address, dest must be at least 16. Returns the
+// number of bytes written in dest, that is 4 for IPv4 and 16 for
+// IPv6. Return 0 if error occurred.
+size_t getBinAddr(void* dest, const std::string& ip);
+
+// Verifies hostname against presented identifiers in the certificate.
+// The implementation is based on the procedure described in RFC 6125.
+bool verifyHostname(const std::string& hostname,
+                    const std::vector<std::string>& dnsNames,
+                    const std::vector<std::string>& ipAddrs,
+                    const std::string& commonName);
+
+} // namespace net
+
 } // namespace aria2
 
-#endif // _D_SOCKET_CORE_H_
+#endif // D_SOCKET_CORE_H
