@@ -38,7 +38,7 @@
 #include "a2functional.h"
 #include "bittorrent_helper.h"
 #include "DlAbortEx.h"
-#include "StringFormat.h"
+#include "fmt.h"
 #include "BtMessageFactory.h"
 #include "BtMessageDispatcher.h"
 #include "Peer.h"
@@ -48,11 +48,17 @@
 #include "DownloadContext.h"
 #include "BtMessage.h"
 #include "PieceStorage.h"
+#include "ExtensionMessageRegistry.h"
 
 namespace aria2 {
 
 UTMetadataRequestExtensionMessage::UTMetadataRequestExtensionMessage
-(uint8_t extensionMessageID):UTMetadataExtensionMessage(extensionMessageID) {}
+(uint8_t extensionMessageID):UTMetadataExtensionMessage(extensionMessageID),
+                             dispatcher_(0),
+                             messageFactory_(0)
+{}
+
+UTMetadataRequestExtensionMessage::~UTMetadataRequestExtensionMessage() {}
 
 std::string UTMetadataRequestExtensionMessage::getPayload()
 {
@@ -64,13 +70,15 @@ std::string UTMetadataRequestExtensionMessage::getPayload()
 
 std::string UTMetadataRequestExtensionMessage::toString() const
 {
-  return strconcat("ut_metadata request piece=", util::uitos(getIndex()));
+  return fmt("ut_metadata request piece=%lu",
+             static_cast<unsigned long>(getIndex()));
 }
 
 void UTMetadataRequestExtensionMessage::doReceivedAction()
 {
   SharedHandle<TorrentAttribute> attrs = bittorrent::getTorrentAttrs(dctx_);
-  uint8_t id = peer_->getExtensionMessageID("ut_metadata");
+  uint8_t id = peer_->getExtensionMessageID
+    (ExtensionMessageRegistry::UT_METADATA);
   if(attrs->metadata.empty()) {
     SharedHandle<UTMetadataRejectExtensionMessage> m
       (new UTMetadataRejectExtensionMessage(id));
@@ -88,15 +96,25 @@ void UTMetadataRequestExtensionMessage::doReceivedAction()
       (getIndex()+1)*METADATA_PIECE_SIZE <= attrs->metadata.size()?
       attrs->metadata.begin()+(getIndex()+1)*METADATA_PIECE_SIZE:
       attrs->metadata.end();
-    m->setData(std::string(begin, end));
+    m->setData(begin, end);
     SharedHandle<BtMessage> msg = messageFactory_->createBtExtendedMessage(m);
     dispatcher_->addMessageToQueue(msg);
   } else {
     throw DL_ABORT_EX
-      (StringFormat
-       ("Metadata piece index is too big. piece=%lu",
-        static_cast<unsigned long>(getIndex())).str());
+      (fmt("Metadata piece index is too big. piece=%lu",
+           static_cast<unsigned long>(getIndex())));
   }
 }
 
+void UTMetadataRequestExtensionMessage::setDownloadContext
+(const SharedHandle<DownloadContext>& dctx)
+{
+  dctx_ = dctx;
+}
+
+void UTMetadataRequestExtensionMessage::setPeer(const SharedHandle<Peer>& peer)
+{
+  peer_ = peer;
+}
+
 } // namespace aria2
